@@ -1,5 +1,5 @@
 /* Test the Unicode character name functions.
-   Copyright (C) 2000-2003, 2005, 2007, 2009-2015 Free Software Foundation,
+   Copyright (C) 2000-2003, 2005, 2007, 2009-2018 Free Software Foundation,
    Inc.
 
    This program is free software: you can redistribute it and/or modify
@@ -13,7 +13,7 @@
    GNU General Public License for more details.
 
    You should have received a copy of the GNU General Public License
-   along with this program.  If not, see <http://www.gnu.org/licenses/>.  */
+   along with this program.  If not, see <https://www.gnu.org/licenses/>.  */
 
 #include <config.h>
 
@@ -23,7 +23,6 @@
 
 #include "xalloc.h"
 #include "uniname.h"
-#include "progname.h"
 
 /* The names according to the UnicodeData.txt file, modified to contain the
    Hangul syllable names, as described in the Unicode 3.0 book.  */
@@ -40,51 +39,19 @@ struct unicode_alias
 };
 
 static struct unicode_alias unicode_aliases [ALIASLEN];
-
-/* Maximum length of a field in the UnicodeData.txt file.  */
-#define FIELDLEN 120
-
-/* Reads the next field from STREAM.  The buffer BUFFER has size FIELDLEN.
-   Reads up to (but excluding) DELIM.
-   Returns 1 when a field was successfully read, otherwise 0.  */
-static int
-getfield (FILE *stream, char *buffer, int delim)
-{
-  int count = 0;
-  int c;
-
-  for (; (c = getc (stream)), (c != EOF && c != delim); )
-    {
-      /* Put c into the buffer.  */
-      if (++count >= FIELDLEN - 1)
-        {
-          fprintf (stderr, "field too long\n");
-          exit (EXIT_FAILURE);
-        }
-      *buffer++ = c;
-    }
-
-  if (c == EOF)
-    return 0;
-
-  *buffer = '\0';
-  return 1;
-}
+static int aliases_count;
 
 /* Stores in unicode_names[] the relevant contents of the UnicodeData.txt
    file.  */
 static void
 fill_names (const char *unicodedata_filename)
 {
-  unsigned int i;
   FILE *stream;
-  char field0[FIELDLEN];
-  char field1[FIELDLEN];
+  char *field0;
+  char *field1;
+  char line[1024];
   int lineno = 0;
 
-  for (i = 0; i < 0x110000; i++)
-    unicode_names[i] = NULL;
-
   stream = fopen (unicodedata_filename, "r");
   if (stream == NULL)
     {
@@ -92,24 +59,41 @@ fill_names (const char *unicodedata_filename)
       exit (EXIT_FAILURE);
     }
 
-  for (;;)
+  while (fgets (line, sizeof line, stream))
     {
-      int n;
-      int c;
+      char *p;
+      char *comment;
+      unsigned long i;
 
       lineno++;
-      n = getfield (stream, field0, ';');
-      n += getfield (stream, field1, ';');
-      if (n == 0)
-        break;
-      if (n != 2)
+
+      comment = strchr (line, '#');
+      if (comment != NULL)
+        *comment = '\0';
+      if (line[strspn (line, " \t\r\n")] == '\0')
+        continue;
+
+      field0 = p = line;
+      p = strchr (p, ';');
+      if (!p)
         {
           fprintf (stderr, "short line in '%s':%d\n",
                    unicodedata_filename, lineno);
           exit (EXIT_FAILURE);
         }
-      for (; (c = getc (stream)), (c != EOF && c != '\n'); )
-        ;
+      *p++ = '\0';
+
+      field1 = p;
+      if (*field1 == '<')
+        continue;
+      p = strchr (p, ';');
+      if (!p)
+        {
+          fprintf (stderr, "short line in '%s':%d\n",
+                   unicodedata_filename, lineno);
+          exit (EXIT_FAILURE);
+        }
+      *p = '\0';
       i = strtoul (field0, NULL, 16);
       if (i >= 0x110000)
         {
@@ -130,15 +114,12 @@ fill_names (const char *unicodedata_filename)
 static void
 fill_aliases (const char *namealiases_filename)
 {
-  int i;
   FILE *stream;
-  char field0[FIELDLEN];
-  char field1[FIELDLEN];
+  char *field0;
+  char *field1;
+  char line[1024];
   int lineno = 0;
 
-  for (i = 0; i < ALIASLEN; i++)
-    unicode_aliases[i].uc = UNINAME_INVALID;
-
   stream = fopen (namealiases_filename, "r");
   if (stream == NULL)
     {
@@ -146,33 +127,55 @@ fill_aliases (const char *namealiases_filename)
       exit (EXIT_FAILURE);
     }
 
-  for (i = 0; i < ALIASLEN; i++)
+  while (fgets (line, sizeof line, stream))
     {
-      int n;
-      int c;
-      unsigned int uc;
+      char *p;
+      char *comment;
+      unsigned long uc;
+
+      comment = strchr (line, '#');
+      if (comment != NULL)
+        *comment = '\0';
+      if (line[strspn (line, " \t\r\n")] == '\0')
+        continue;
 
       lineno++;
-      n = getfield (stream, field0, ';');
-      n += getfield (stream, field1, ';');
-      if (n == 0)
-        break;
-      if (n != 2)
+
+      field0 = p = line;
+      p = strchr (p, ';');
+      if (!p)
         {
           fprintf (stderr, "short line in '%s':%d\n",
                    namealiases_filename, lineno);
           exit (EXIT_FAILURE);
         }
-      for (; (c = getc (stream)), (c != EOF && c != '\n'); )
-        ;
+      *p++ = '\0';
+
+      field1 = p;
+      p = strchr (p, ';');
+      if (!p)
+        {
+          fprintf (stderr, "short line in '%s':%d\n",
+                   namealiases_filename, lineno);
+          exit (EXIT_FAILURE);
+        }
+      *p = '\0';
+
       uc = strtoul (field0, NULL, 16);
       if (uc >= 0x110000)
         {
           fprintf (stderr, "index too large\n");
           exit (EXIT_FAILURE);
         }
-      unicode_aliases[i].name = xstrdup (field1);
-      unicode_aliases[i].uc = uc;
+
+      if (aliases_count == ALIASLEN)
+        {
+          fprintf (stderr, "too many aliases\n");
+          exit (EXIT_FAILURE);
+        }
+      unicode_aliases[aliases_count].name = xstrdup (field1);
+      unicode_aliases[aliases_count].uc = uc;
+      aliases_count++;
     }
   if (ferror (stream) || fclose (stream))
     {
@@ -181,6 +184,16 @@ fill_aliases (const char *namealiases_filename)
     }
 }
 
+static int
+name_has_alias (unsigned int uc)
+{
+  int i;
+  for (i = 0; i < ALIASLEN; i++)
+    if (unicode_aliases[i].uc == uc)
+      return 1;
+  return 0;
+}
+
 /* Perform an exhaustive test of the unicode_character_name function.  */
 static int
 test_name_lookup ()
@@ -296,6 +309,7 @@ test_inverse_lookup ()
 
                 result = unicode_name_character (buf);
                 if (result != UNINAME_INVALID
+                    && !name_has_alias (result)
                     && !(unicode_names[result] != NULL
                          && strcmp (unicode_names[result], buf) == 0))
                   {
@@ -334,11 +348,11 @@ test_alias_lookup ()
           {
             if (result == UNINAME_INVALID)
               fprintf (stderr, "inverse name lookup of \"%s\" failed\n",
-                       unicode_aliases[i]);
+                       unicode_aliases[i].name);
             else
               fprintf (stderr,
                        "inverse name lookup of \"%s\" returned 0x%04X\n",
-                       unicode_aliases[i], result);
+                       unicode_aliases[i].name, result);
             error = 1;
           }
       }
@@ -350,19 +364,27 @@ int
 main (int argc, char *argv[])
 {
   int error = 0;
+  int i;
 
-  set_program_name (argv[0]);
+  for (i = 1; i < argc && strcmp (argv[i], "--") != 0; i++)
+    fill_names (argv[i]);
 
-  fill_names (argv[1]);
+  if (i < argc)
+    {
+      int j;
+      for (j = 0; j < ALIASLEN; j++)
+        unicode_aliases[j].uc = UNINAME_INVALID;
+
+      i++;
+      for (; i < argc; i++)
+        fill_aliases (argv[i]);
+    }
 
   error |= test_name_lookup ();
   error |= test_inverse_lookup ();
 
-  if (argc > 2)
-    {
-      fill_aliases (argv[2]);
-      error |= test_alias_lookup ();
-    }
+  if (aliases_count > 0)
+    error |= test_alias_lookup ();
 
   return error;
 }
